#include "contours_lib.h"
#include <opencv2/opencv.hpp>

static std::vector<std::vector<cv::Point>> contours;

std::vector<std::vector<std::array<int, 2>>>
get_contours(const int width, const int height,
             const std::vector<float> data) {
    cv::Mat img(height, width, CV_8UC3);  // (1)
    for (int y = 0; y < height; y++)
        for (int x = 0; x < width; x++) {
            const int index = y * width + x;
            img.at<cv::Vec3b>(y, x) =
            cv::Vec3b(
                      (uchar)floor(data[index * 3 + 2] * 255),
                      (uchar)floor(data[index * 3 + 1] * 255),
                      (uchar)floor(data[index * 3 + 0] * 255));
        } // (2)
    
    cv::Mat img_;
    cv::cvtColor(img, img_, cv::COLOR_RGB2GRAY); // (3)
    cv::threshold(img_, img_, 127, 255, cv::THRESH_BINARY); // (4)
    
    cv::findContours(img_, contours,
                     cv::RETR_TREE, cv::CHAIN_APPROX_SIMPLE); // (5)
    
    std::vector<std::vector<std::array<int, 2>>> result;
    for (auto cont : contours) {
        std::vector<std::array<int, 2>> pts_;
        for (auto pt : cont)
            pts_.push_back({ pt.y, pt.x });
        result.push_back(pts_); // (6)
    }
    return result;
}
