// Copyright 1998-2017 Epic Games, Inc. All Rights Reserved.

#include "ActorLocCSV.h"
#include "ActorLocCSVStyle.h"
#include "ActorLocCSVCommands.h"
#include "Misc/MessageDialog.h"
#include "Framework/MultiBox/MultiBoxBuilder.h"
#include "LevelEditor.h"

#include "ActorLocUpdater.h"
#include "Editor/UnrealEd/Classes/Editor/EditorEngine.h"
#include "UnrealEd.h"


static const FName ActorLocCSVTabName("ActorLocCSV");

#define LOCTEXT_NAMESPACE "FActorLocCSVModule"

void FActorLocCSVModule::StartupModule()
{
	// This code will execute after your module is loaded into memory; the exact timing is specified in the .uplugin file per-module
	
	FActorLocCSVStyle::Initialize();
	FActorLocCSVStyle::ReloadTextures();

	FActorLocCSVCommands::Register();
	
	PluginCommands = MakeShareable(new FUICommandList);

	PluginCommands->MapAction(
		FActorLocCSVCommands::Get().PluginAction,
		FExecuteAction::CreateRaw(this, &FActorLocCSVModule::PluginButtonClicked),
		FCanExecuteAction());
		
	FLevelEditorModule& LevelEditorModule = FModuleManager::LoadModuleChecked<FLevelEditorModule>("LevelEditor");
	
	{
		TSharedPtr<FExtender> MenuExtender = MakeShareable(new FExtender());
		MenuExtender->AddMenuExtension("WindowLayout", EExtensionHook::After, PluginCommands, FMenuExtensionDelegate::CreateRaw(this, &FActorLocCSVModule::AddMenuExtension));

		LevelEditorModule.GetMenuExtensibilityManager()->AddExtender(MenuExtender);
	}
	
	{
		TSharedPtr<FExtender> ToolbarExtender = MakeShareable(new FExtender);
		ToolbarExtender->AddToolBarExtension("Settings", EExtensionHook::After, PluginCommands, FToolBarExtensionDelegate::CreateRaw(this, &FActorLocCSVModule::AddToolbarExtension));
		
		LevelEditorModule.GetToolBarExtensibilityManager()->AddExtender(ToolbarExtender);
	}
}

void FActorLocCSVModule::ShutdownModule()
{
	// This function may be called during shutdown to clean up your module.  For modules that support dynamic reloading,
	// we call this function before unloading the module.
	FActorLocCSVStyle::Shutdown();

	FActorLocCSVCommands::Unregister();
}

void FActorLocCSVModule::PluginButtonClicked()
{
    UWorld* World = GEditor->GetEditorWorldContext().World();
    FString FilePath = ActorLocCSV::ActorLocUpdater(World).Update();
    FText Message;
    if (FilePath != "") {
        Message = FText::FromString("Updated to " + FilePath + ".");
    } else {
        Message = FText::FromString("Failed to save a CSV file.");
    }
	FMessageDialog::Open(EAppMsgType::Ok, Message);
}

void FActorLocCSVModule::AddMenuExtension(FMenuBuilder& Builder)
{
	Builder.AddMenuEntry(FActorLocCSVCommands::Get().PluginAction);
}

void FActorLocCSVModule::AddToolbarExtension(FToolBarBuilder& Builder)
{
	Builder.AddToolBarButton(FActorLocCSVCommands::Get().PluginAction);
}

#undef LOCTEXT_NAMESPACE
	
IMPLEMENT_MODULE(FActorLocCSVModule, ActorLocCSV)