#include "ActorLocDiff.h"

using namespace ActorLocCSV;

// constructors
ActorLocDiff::ActorLocDiff(const FString& Label) // (S1)
  : Label(Label), Updated(false) {}

ActorLocDiff::ActorLocDiff(const FString& Label,
			   const TArray<FString>& Items)  // (S2)
  : Label(Label) {
  const auto vals = FloatsOf(Items); // (S3)
  if (vals.Num() >= 7) { // (S4)
    LastLocation = FVectorOf(vals, 1);
    LastRotation = FRotatorOf(vals, 4);
  }
  if (vals.Num() >= 13) { // (S5)
    NewLocation = FVectorOf(vals, 7);
    NewRotation = FRotatorOf(vals, 10);
    Updated = true;
  } else {
    Updated = false;
  }
}

// public method
auto ActorLocDiff::ItemStrings(void) const // (S6)
  -> TArray<FString> {
  TArray<FString> strs;
  strs.Emplace(Label); // (S7)
  strs += StrsOf(FloatsOf(LastLocation)); // (S8)
  strs += StrsOf(FloatsOf(LastRotation)); // (S9)
  // Note that NewLocation, NewRotation are not written
  // to the CSV file.
  return strs;
}

// private methods
auto ActorLocDiff::StrsOf(const TArray<float>& Floats)
  -> TArray<FString> {
  TArray<FString> strs;
  for (const float val : Floats)
    strs.Emplace(FString::SanitizeFloat(val));
  return strs;
}

auto ActorLocDiff::FloatsOf(const TArray<FString>& Strs)
  -> TArray<float> {
  TArray<float> floats;
  for (const FString& str : Strs)
    if (str == "")
      return floats;
    else if (str.IsNumeric())
      floats.Emplace(FCString::Atof(*str));
    else
      floats.Emplace(0.0f);
  return floats;
}

auto ActorLocDiff::FloatsOf(const FVector& Vector)
  -> TArray<float> {
  return TArray<float>({Vector.X, Vector.Y, Vector.Z});
}

auto ActorLocDiff::FloatsOf(const FRotator& Rotator)
  -> TArray<float> {
  return TArray<float>(
       {Rotator.Pitch, Rotator.Yaw, Rotator.Roll});
}

auto ActorLocDiff::FVectorOf(const TArray<float>& Floats,
			     const int Index)
  -> FVector {
  return FVector(Floats[Index + 0],
		 Floats[Index + 1],
		 Floats[Index + 2]);
}

auto ActorLocDiff::FRotatorOf(const TArray<float>& Floats,
			      const int Index)
  -> FRotator {
  return FRotator(Floats[Index + 0], // pitch
		  Floats[Index + 1], // yaw
		  Floats[Index + 2]); // roll
}
