#include "ActorLocModel.h"
#include <fstream>

using namespace ActorLocCSV;

// public methods
auto ActorLocModel::Read(void)
  -> bool {
  ActorLocDiffs.Empty();
  std::ifstream ifs(filepath());
  if (!ifs) return false;
  std::string line;
  std::getline(ifs, line); // skip the header
  while (std::getline(ifs, line)) {
    FString Line(line.c_str());
    TArray<FString> Items;
    Line.ParseIntoArray(Items, TEXT(","), true);
    if (Items.Num() > 0)
      ActorLocDiffs.Emplace(ActorLocDiff(Items[0], Items));
  }
  return true;
}

auto ActorLocModel::Write(void) const
  -> bool {
  std::ofstream ofs(filepath());
  if (!ofs) return false;
  ofs << "Label,LastX,LastY,LastZ,"
    "LastPitch,LastYaw,LastRoll,"
    "NewX,NewY,NewZ,"
    "NewPitch,NewYaw,NewRoll" << std::endl;
  for (const auto& locdiff : ActorLocDiffs) {
      for (const auto& item : locdiff.ItemStrings()) {
          std::string str(TCHAR_TO_UTF8(*item));
          ofs << str << ",";
      }
    ofs << std::endl;
  }
  return true;
}

auto ActorLocModel::operator[](const FString& Label)
  -> ActorLocDiff& {
  for (auto& locdiff : ActorLocDiffs)
    if (locdiff.Label == Label) return locdiff;
  ActorLocDiffs.Emplace(ActorLocDiff(Label));
  return ActorLocDiffs.Last();
}
