#include "ActorLocUpdater.h"
#include "ActorLocModel.h"
#include "EngineUtils.h"
#include "Runtime/Engine/Classes/Engine/StaticMeshActor.h"
#include "Runtime/Core/Public/Misc/Paths.h"

using namespace ActorLocCSV;

ActorLocUpdater::ActorLocUpdater(UWorld* World)
  : MyWorld(World) {
    const FString rel_path = FPaths::ProjectDir() /
        World->GetMapName() + "_ActorLoc.csv";
    const FString abs_path =
        FPaths::ConvertRelativePathToFull(rel_path);
    Model = MakeShareable(new ActorLocModel(abs_path));
}

auto ActorLocUpdater::Update(void)
  -> FString {
  Model->Read();
  for (TActorIterator<AStaticMeshActor> actorIt(MyWorld);
       actorIt; ++actorIt) {
    auto& loc_diff = (*Model)[actorIt->GetActorLabel()];
    if (loc_diff.Updated) {
      actorIt->SetActorLocation(loc_diff.NewLocation);
      actorIt->SetActorRotation(loc_diff.NewRotation);
    }
    loc_diff.LastLocation = actorIt->GetActorLocation();
    loc_diff.LastRotation = actorIt->GetActorRotation();
  }
  if (Model->Write())
      return Model->Filepath;
  else
      return "";
}
