#include "ContoursCaptureComponent2D.h"
#include "contours_lib.h"
#include <vector>

// constructor

UContoursCaptureComponent2D::UContoursCaptureComponent2D(void) : Super() {
    bCaptureEveryFrame = false;
}

// public methods
auto UContoursCaptureComponent2D::NumberOfContours(void)
-> int32 {  // (S1)
    TextureTarget = NewObject<UTextureRenderTarget2D>(this);  // (S2)
    if (CanvasSize.X < 10) CanvasSize.X = 10; // (S3)
    if (CanvasSize.Y < 10) CanvasSize.Y = 10;
    CanvasSize.X = floor(CanvasSize.X);
    CanvasSize.Y = floor(CanvasSize.Y);
    TextureTarget->InitAutoFormat(CanvasSize.X, CanvasSize.Y); // (S4)

    CaptureSource = ESceneCaptureSource::SCS_FinalColorLDR;
    CaptureScene();  // (S5)

    FReadSurfaceDataFlags ReadPixelFlags(RCM_UNorm);
    FTextureRenderTargetResource* RTResource =
        TextureTarget->GameThread_GetRenderTargetResource();
    
    TArray<FColor> colors;
    const bool result = RTResource->ReadPixels(colors); // (S6)

    if (!result || colors.Num() == 0) return 0;
    CalculateContours(colors); // (S7)
    return Contours.Num(); // (S8)
}

auto UContoursCaptureComponent2D::ContourOfIndex(const int32 Index)
-> TArray<FVector2D> { // (S9)
  if (Index < 0 || Index >= Contours.Num())
    return TArray<FVector2D>();
  return Contours[Index]; // (S10)
}

// private methods
auto UContoursCaptureComponent2D::CalculateContours(
    const TArray<FColor>& Colors)
-> void { // (S11)
    std::vector<float> data; // (S12)
    data.reserve(Colors.Num() * 3);
    for (const auto& col : Colors) {
        data.emplace_back(col.R);
        data.emplace_back(col.G);
        data.emplace_back(col.B);
    }

    auto contours_ = get_contours(
        (int)(CanvasSize.X),
        (int)(CanvasSize.Y),
        data); // (S13)

    Contours.Empty(); // (S14)
    for (const auto& cont_ : contours_) {
        TArray<FVector2D> cont;
        for (const auto& pt_ : cont_)
            cont.Emplace(FVector2D(
                pt_[1] / CanvasSize.X,
                pt_[0] / CanvasSize.Y)); // (S15)
        Contours.Emplace(cont);
    }
}
