// Copyright 1998-2017 Epic Games, Inc. All Rights Reserved.

#include "MeshEdit.h"
#include "MeshEditStyle.h"
#include "MeshEditCommands.h"
#include "Misc/MessageDialog.h"
#include "Framework/MultiBox/MultiBoxBuilder.h"

#include "LevelEditor.h"

// (1) from here
#include "MeshEditMode.h"
#include "Editor/UnrealEd/Public/EditorModeManager.h"
// (1) till here

static const FName MeshEditTabName("MeshEdit");

#define LOCTEXT_NAMESPACE "FMeshEditModule"

void FMeshEditModule::StartupModule()
{
	// This code will execute after your module is loaded into memory; the exact timing is specified in the .uplugin file per-module
	
	FMeshEditStyle::Initialize();
	FMeshEditStyle::ReloadTextures();

	FMeshEditCommands::Register();
	
	PluginCommands = MakeShareable(new FUICommandList);

	PluginCommands->MapAction(
		FMeshEditCommands::Get().PluginAction,
		FExecuteAction::CreateRaw(this, &FMeshEditModule::PluginButtonClicked),
		FCanExecuteAction());
		
	FLevelEditorModule& LevelEditorModule = FModuleManager::LoadModuleChecked<FLevelEditorModule>("LevelEditor");
	
	{
		TSharedPtr<FExtender> MenuExtender = MakeShareable(new FExtender());
		MenuExtender->AddMenuExtension("WindowLayout", EExtensionHook::After, PluginCommands, FMenuExtensionDelegate::CreateRaw(this, &FMeshEditModule::AddMenuExtension));

		LevelEditorModule.GetMenuExtensibilityManager()->AddExtender(MenuExtender);
	}
	
	{
		TSharedPtr<FExtender> ToolbarExtender = MakeShareable(new FExtender);
		ToolbarExtender->AddToolBarExtension("Settings", EExtensionHook::After, PluginCommands, FToolBarExtensionDelegate::CreateRaw(this, &FMeshEditModule::AddToolbarExtension));
		
		LevelEditorModule.GetToolBarExtensibilityManager()->AddExtender(ToolbarExtender);
	}
}

void FMeshEditModule::ShutdownModule()
{
	// This function may be called during shutdown to clean up your module.  For modules that support dynamic reloading,
	// we call this function before unloading the module.
	FMeshEditStyle::Shutdown();

	FMeshEditCommands::Unregister();
}

void FMeshEditModule::PluginButtonClicked() { // (2)
    FMeshEditMode::Register(); // (3)
    if (GLevelEditorModeTools().
	IsModeActive(FMeshEditMode::MeshEditModeID)) { // (4)
        GLevelEditorModeTools().
	    DeactivateMode(FMeshEditMode::MeshEditModeID);
    } else {
        GLevelEditorModeTools().
	    ActivateMode(FMeshEditMode::MeshEditModeID);
    }
}

void FMeshEditModule::AddMenuExtension(FMenuBuilder& Builder)
{
	Builder.AddMenuEntry(FMeshEditCommands::Get().PluginAction);
}

void FMeshEditModule::AddToolbarExtension(FToolBarBuilder& Builder)
{
	Builder.AddToolBarButton(FMeshEditCommands::Get().PluginAction);
}

#undef LOCTEXT_NAMESPACE
	
IMPLEMENT_MODULE(FMeshEditModule, MeshEdit)
